@@ -6,6 +6,7 @@ and tools.
 from pydantic import BaseModel
 
 from mcp.types import (
+    Icon,
     ServerCapabilities,
 )
 
@@ -15,3 +16,5 @@ class InitializationOptions(BaseModel):
     server_version: str
     capabilities: ServerCapabilities
     instructions: str | None = None
+    website_url: str | None = None
+    icons: list[Icon] | None = None
