@@ -1,3 +1,4 @@
+import errno
 import os
 import shutil
 import sys
@@ -9,28 +10,25 @@ import anyio
 import pytest
 
 from mcp.client.session import ClientSession
-from mcp.client.stdio import (
-    StdioServerParameters,
-    _create_platform_compatible_process,
-    stdio_client,
-)
+from mcp.client.stdio import StdioServerParameters, _create_platform_compatible_process, stdio_client
 from mcp.shared.exceptions import McpError
 from mcp.shared.message import SessionMessage
 from mcp.types import CONNECTION_CLOSED, JSONRPCMessage, JSONRPCRequest, JSONRPCResponse
-from tests.shared.test_win32_utils import escape_path_for_python
+
+from ..shared.test_win32_utils import escape_path_for_python
 
 # Timeout for cleanup of processes that ignore SIGTERM
 # This timeout ensures the test fails quickly if the cleanup logic doesn't have
 # proper fallback mechanisms (SIGINT/SIGKILL) for processes that ignore SIGTERM
 SIGTERM_IGNORING_PROCESS_TIMEOUT = 5.0
 
-tee: str = shutil.which("tee")  # type: ignore
-python: str = shutil.which("python")  # type: ignore
+tee = shutil.which("tee")
 
 
 @pytest.mark.anyio
 @pytest.mark.skipif(tee is None, reason="could not find tee command")
 async def test_stdio_context_manager_exiting():
+    assert tee is not None
     async with stdio_client(StdioServerParameters(command=tee)) as (_, _):
         pass
 
@@ -38,6 +36,7 @@ async def test_stdio_context_manager_exiting():
 @pytest.mark.anyio
 @pytest.mark.skipif(tee is None, reason="could not find tee command")
 async def test_stdio_client():
+    assert tee is not None
     server_parameters = StdioServerParameters(command=tee)
 
     async with stdio_client(server_parameters) as (read_stream, write_stream):
@@ -52,10 +51,10 @@ async def test_stdio_client():
                 session_message = SessionMessage(message)
                 await write_stream.send(session_message)
 
-        read_messages = []
+        read_messages: list[JSONRPCMessage] = []
         async with read_stream:
             async for message in read_stream:
-                if isinstance(message, Exception):
+                if isinstance(message, Exception):  # pragma: no cover
                     raise message
 
                 read_messages.append(message.message)
@@ -92,17 +91,12 @@ async def test_stdio_client_nonexistent_command():
     )
 
     # Should raise an error when trying to start the process
-    with pytest.raises(Exception) as exc_info:
+    with pytest.raises(OSError) as exc_info:
         async with stdio_client(server_params) as (_, _):
-            pass
+            pass  # pragma: no cover
 
-    # The error should indicate the command was not found
-    error_message = str(exc_info.value)
-    assert (
-        "nonexistent" in error_message
-        or "not found" in error_message.lower()
-        or "cannot find the file" in error_message.lower()  # Windows error message
-    )
+    # The error should indicate the command was not found (ENOENT: No such file or directory)
+    assert exc_info.value.errno == errno.ENOENT
 
 
 @pytest.mark.anyio
@@ -118,7 +112,7 @@ async def test_stdio_client_universal_cleanup():
         """
         import time
         import sys
-        
+
         # Simulate a long-running process
         for i in range(100):
             time.sleep(0.1)
@@ -136,7 +130,7 @@ async def test_stdio_client_universal_cleanup():
     start_time = time.time()
 
     with anyio.move_on_after(8.0) as cancel_scope:
-        async with stdio_client(server_params) as (read_stream, write_stream):
+        async with stdio_client(server_params) as (_, _):
             # Immediately exit - this triggers cleanup while process is still running
             pass
 
@@ -150,7 +144,7 @@ async def test_stdio_client_universal_cleanup():
         )
 
     # Check if we timed out
-    if cancel_scope.cancelled_caught:
+    if cancel_scope.cancelled_caught:  # pragma: no cover
         pytest.fail(
             "stdio_client cleanup timed out after 8.0 seconds. "
             "This indicates the cleanup mechanism is hanging and needs fixing."
@@ -159,7 +153,7 @@ async def test_stdio_client_universal_cleanup():
 
 @pytest.mark.anyio
 @pytest.mark.skipif(sys.platform == "win32", reason="Windows signal handling is different")
-async def test_stdio_client_sigint_only_process():
+async def test_stdio_client_sigint_only_process():  # pragma: no cover
     """
     Test cleanup with a process that ignores SIGTERM but responds to SIGINT.
     """
@@ -195,13 +189,13 @@ async def test_stdio_client_sigint_only_process():
     try:
         # Use anyio timeout to prevent test from hanging forever
         with anyio.move_on_after(5.0) as cancel_scope:
-            async with stdio_client(server_params) as (read_stream, write_stream):
+            async with stdio_client(server_params) as (_, _):
                 # Let the process start and begin ignoring SIGTERM
                 await anyio.sleep(0.5)
                 # Exit context triggers cleanup - this should not hang
                 pass
 
-        if cancel_scope.cancelled_caught:
+        if cancel_scope.cancelled_caught:  # pragma: no cover
             raise TimeoutError("Test timed out")
 
         end_time = time.time()
@@ -214,7 +208,7 @@ async def test_stdio_client_sigint_only_process():
             f"Expected < {SIGTERM_IGNORING_PROCESS_TIMEOUT} seconds. "
             "This suggests the cleanup needs SIGINT/SIGKILL fallback."
         )
-    except (TimeoutError, Exception) as e:
+    except (TimeoutError, Exception) as e:  # pragma: no cover
         if isinstance(e, TimeoutError) or "timed out" in str(e):
             pytest.fail(
                 f"stdio_client cleanup timed out after {SIGTERM_IGNORING_PROCESS_TIMEOUT} seconds "
@@ -309,7 +303,7 @@ class TestChildProcessCleanup:
             assert os.path.exists(parent_marker), "Parent process didn't start"
 
             # Verify child is writing
-            if os.path.exists(marker_file):
+            if os.path.exists(marker_file):  # pragma: no branch
                 initial_size = os.path.getsize(marker_file)
                 await anyio.sleep(0.3)
                 size_after_wait = os.path.getsize(marker_file)
@@ -324,7 +318,7 @@ class TestChildProcessCleanup:
 
             # Verify processes stopped
             await anyio.sleep(0.5)
-            if os.path.exists(marker_file):
+            if os.path.exists(marker_file):  # pragma: no branch
                 size_after_cleanup = os.path.getsize(marker_file)
                 await anyio.sleep(0.5)
                 final_size = os.path.getsize(marker_file)
@@ -341,7 +335,7 @@ class TestChildProcessCleanup:
             for f in [marker_file, parent_marker]:
                 try:
                     os.unlink(f)
-                except OSError:
+                except OSError:  # pragma: no cover
                     pass
 
     @pytest.mark.anyio
@@ -412,7 +406,7 @@ class TestChildProcessCleanup:
 
             # Verify all are writing
             for file_path, name in [(parent_file, "parent"), (child_file, "child"), (grandchild_file, "grandchild")]:
-                if os.path.exists(file_path):
+                if os.path.exists(file_path):  # pragma: no branch
                     initial_size = os.path.getsize(file_path)
                     await anyio.sleep(0.3)
                     new_size = os.path.getsize(file_path)
@@ -426,7 +420,7 @@ class TestChildProcessCleanup:
             # Verify all stopped
             await anyio.sleep(0.5)
             for file_path, name in [(parent_file, "parent"), (child_file, "child"), (grandchild_file, "grandchild")]:
-                if os.path.exists(file_path):
+                if os.path.exists(file_path):  # pragma: no branch
                     size1 = os.path.getsize(file_path)
                     await anyio.sleep(0.3)
                     size2 = os.path.getsize(file_path)
@@ -439,7 +433,7 @@ class TestChildProcessCleanup:
             for f in [parent_file, child_file, grandchild_file]:
                 try:
                     os.unlink(f)
-                except OSError:
+                except OSError:  # pragma: no cover
                     pass
 
     @pytest.mark.anyio
@@ -493,7 +487,7 @@ class TestChildProcessCleanup:
             await anyio.sleep(0.5)
 
             # Verify child is writing
-            if os.path.exists(marker_file):
+            if os.path.exists(marker_file):  # pragma: no cover
                 size1 = os.path.getsize(marker_file)
                 await anyio.sleep(0.3)
                 size2 = os.path.getsize(marker_file)
@@ -506,7 +500,7 @@ class TestChildProcessCleanup:
 
             # Verify child stopped
             await anyio.sleep(0.5)
-            if os.path.exists(marker_file):
+            if os.path.exists(marker_file):  # pragma: no branch
                 size3 = os.path.getsize(marker_file)
                 await anyio.sleep(0.3)
                 size4 = os.path.getsize(marker_file)
@@ -518,7 +512,7 @@ class TestChildProcessCleanup:
             # Clean up marker file
             try:
                 os.unlink(marker_file)
-            except OSError:
+            except OSError:  # pragma: no cover
                 pass
 
 
@@ -532,7 +526,7 @@ async def test_stdio_client_graceful_stdin_exit():
     script_content = textwrap.dedent(
         """
         import sys
-        
+
         # Read from stdin until it's closed
         try:
             while True:
@@ -541,7 +535,7 @@ async def test_stdio_client_graceful_stdin_exit():
                     break
         except:
             pass
-        
+
         # Exit gracefully
         sys.exit(0)
         """
@@ -556,7 +550,7 @@ async def test_stdio_client_graceful_stdin_exit():
 
     # Use anyio timeout to prevent test from hanging forever
     with anyio.move_on_after(5.0) as cancel_scope:
-        async with stdio_client(server_params) as (read_stream, write_stream):
+        async with stdio_client(server_params) as (_, _):
             # Let the process start and begin reading stdin
             await anyio.sleep(0.2)
             # Exit context triggers cleanup - process should exit from stdin closure
@@ -566,7 +560,7 @@ async def test_stdio_client_graceful_stdin_exit():
         pytest.fail(
             "stdio_client cleanup timed out after 5.0 seconds. "
             "Process should have exited gracefully when stdin was closed."
-        )
+        )  # pragma: no cover
 
     end_time = time.time()
     elapsed = end_time - start_time
@@ -590,16 +584,16 @@ async def test_stdio_client_stdin_close_ignored():
         import signal
         import sys
         import time
-        
+
         # Set up SIGTERM handler to exit cleanly
         def sigterm_handler(signum, frame):
             sys.exit(0)
-        
+
         signal.signal(signal.SIGTERM, sigterm_handler)
-        
+
         # Close stdin immediately to simulate ignoring it
         sys.stdin.close()
-        
+
         # Keep running until SIGTERM
         while True:
             time.sleep(0.1)
@@ -615,7 +609,7 @@ async def test_stdio_client_stdin_close_ignored():
 
     # Use anyio timeout to prevent test from hanging forever
     with anyio.move_on_after(7.0) as cancel_scope:
-        async with stdio_client(server_params) as (read_stream, write_stream):
+        async with stdio_client(server_params) as (_, _):
             # Let the process start
             await anyio.sleep(0.2)
             # Exit context triggers cleanup
@@ -625,7 +619,7 @@ async def test_stdio_client_stdin_close_ignored():
         pytest.fail(
             "stdio_client cleanup timed out after 7.0 seconds. "
             "Process should have been terminated via SIGTERM escalation."
-        )
+        )  # pragma: no cover
 
     end_time = time.time()
     elapsed = end_time - start_time
