@@ -11,16 +11,8 @@ from starlette.datastructures import Headers
 from starlette.requests import Request
 from starlette.types import Message, Receive, Scope, Send
 
-from mcp.server.auth.middleware.bearer_auth import (
-    AuthenticatedUser,
-    BearerAuthBackend,
-    RequireAuthMiddleware,
-)
-from mcp.server.auth.provider import (
-    AccessToken,
-    OAuthAuthorizationServerProvider,
-    ProviderTokenVerifier,
-)
+from mcp.server.auth.middleware.bearer_auth import AuthenticatedUser, BearerAuthBackend, RequireAuthMiddleware
+from mcp.server.auth.provider import AccessToken, OAuthAuthorizationServerProvider, ProviderTokenVerifier
 
 
 class MockOAuthProvider:
@@ -31,7 +23,7 @@ class MockOAuthProvider:
     """
 
     def __init__(self):
-        self.tokens = {}  # token -> AccessToken
+        self.tokens: dict[str, AccessToken] = {}  # token -> AccessToken
 
     def add_token(self, token: str, access_token: AccessToken) -> None:
         """Add a token to the provider."""
@@ -284,10 +276,10 @@ class TestRequireAuthMiddleware:
         scope: Scope = {"type": "http"}
 
         # Create dummy async functions for receive and send
-        async def receive() -> Message:
+        async def receive() -> Message:  # pragma: no cover
             return {"type": "http.request"}
 
-        sent_messages = []
+        sent_messages: list[Message] = []
 
         async def send(message: Message) -> None:
             sent_messages.append(message)
@@ -308,10 +300,10 @@ class TestRequireAuthMiddleware:
         scope: Scope = {"type": "http", "user": object()}
 
         # Create dummy async functions for receive and send
-        async def receive() -> Message:
+        async def receive() -> Message:  # pragma: no cover
             return {"type": "http.request"}
 
-        sent_messages = []
+        sent_messages: list[Message] = []
 
         async def send(message: Message) -> None:
             sent_messages.append(message)
@@ -337,10 +329,10 @@ class TestRequireAuthMiddleware:
         scope: Scope = {"type": "http", "user": user, "auth": auth}
 
         # Create dummy async functions for receive and send
-        async def receive() -> Message:
+        async def receive() -> Message:  # pragma: no cover
             return {"type": "http.request"}
 
-        sent_messages = []
+        sent_messages: list[Message] = []
 
         async def send(message: Message) -> None:
             sent_messages.append(message)
@@ -365,10 +357,10 @@ class TestRequireAuthMiddleware:
         scope: Scope = {"type": "http", "user": user}  # No auth credentials
 
         # Create dummy async functions for receive and send
-        async def receive() -> Message:
+        async def receive() -> Message:  # pragma: no cover
             return {"type": "http.request"}
 
-        sent_messages = []
+        sent_messages: list[Message] = []
 
         async def send(message: Message) -> None:
             sent_messages.append(message)
@@ -394,10 +386,10 @@ class TestRequireAuthMiddleware:
         scope: Scope = {"type": "http", "user": user, "auth": auth}
 
         # Create dummy async functions for receive and send
-        async def receive() -> Message:
+        async def receive() -> Message:  # pragma: no cover
             return {"type": "http.request"}
 
-        async def send(message: Message) -> None:
+        async def send(message: Message) -> None:  # pragma: no cover
             pass
 
         await middleware(scope, receive, send)
@@ -419,10 +411,10 @@ class TestRequireAuthMiddleware:
         scope: Scope = {"type": "http", "user": user, "auth": auth}
 
         # Create dummy async functions for receive and send
-        async def receive() -> Message:
+        async def receive() -> Message:  # pragma: no cover
             return {"type": "http.request"}
 
-        async def send(message: Message) -> None:
+        async def send(message: Message) -> None:  # pragma: no cover
             pass
 
         await middleware(scope, receive, send)
@@ -444,10 +436,10 @@ class TestRequireAuthMiddleware:
         scope: Scope = {"type": "http", "user": user, "auth": auth}
 
         # Create dummy async functions for receive and send
-        async def receive() -> Message:
+        async def receive() -> Message:  # pragma: no cover
             return {"type": "http.request"}
 
-        async def send(message: Message) -> None:
+        async def send(message: Message) -> None:  # pragma: no cover
             pass
 
         await middleware(scope, receive, send)
