@@ -7,6 +7,7 @@ import hashlib
 import secrets
 import time
 import unittest.mock
+from typing import Any
 from urllib.parse import parse_qs, urlparse
 
 import httpx
@@ -22,34 +23,29 @@ from mcp.server.auth.provider import (
     RefreshToken,
     construct_redirect_uri,
 )
-from mcp.server.auth.routes import (
-    ClientRegistrationOptions,
-    RevocationOptions,
-    create_auth_routes,
-)
-from mcp.shared.auth import (
-    OAuthClientInformationFull,
-    OAuthToken,
-)
+from mcp.server.auth.routes import ClientRegistrationOptions, RevocationOptions, create_auth_routes
+from mcp.shared.auth import OAuthClientInformationFull, OAuthToken
 
 
 # Mock OAuth provider for testing
-class MockOAuthProvider(OAuthAuthorizationServerProvider):
+class MockOAuthProvider(OAuthAuthorizationServerProvider[AuthorizationCode, RefreshToken, AccessToken]):
     def __init__(self):
-        self.clients = {}
-        self.auth_codes = {}  # code -> {client_id, code_challenge, redirect_uri}
-        self.tokens = {}  # token -> {client_id, scopes, expires_at}
-        self.refresh_tokens = {}  # refresh_token -> access_token
+        self.clients: dict[str, OAuthClientInformationFull] = {}
+        self.auth_codes: dict[str, AuthorizationCode] = {}  # code -> {client_id, code_challenge, redirect_uri}
+        self.tokens: dict[str, AccessToken] = {}  # token -> {client_id, scopes, expires_at}
+        self.refresh_tokens: dict[str, str] = {}  # refresh_token -> access_token
 
     async def get_client(self, client_id: str) -> OAuthClientInformationFull | None:
         return self.clients.get(client_id)
 
     async def register_client(self, client_info: OAuthClientInformationFull):
+        assert client_info.client_id is not None
         self.clients[client_info.client_id] = client_info
 
     async def authorize(self, client: OAuthClientInformationFull, params: AuthorizationParams) -> str:
         # toy authorize implementation which just immediately generates an authorization
         # code and completes the redirect
+        assert client.client_id is not None
         code = AuthorizationCode(
             code=f"code_{int(time.time())}",
             client_id=client.client_id,
@@ -78,6 +74,7 @@ class MockOAuthProvider(OAuthAuthorizationServerProvider):
         refresh_token = f"refresh_{secrets.token_hex(32)}"
 
         # Store the tokens
+        assert client.client_id is not None
         self.tokens[access_token] = AccessToken(
             token=access_token,
             client_id=client.client_id,
@@ -103,7 +100,7 @@ class MockOAuthProvider(OAuthAuthorizationServerProvider):
         if old_access_token is None:
             return None
         token_info = self.tokens.get(old_access_token)
-        if token_info is None:
+        if token_info is None:  # pragma: no cover
             return None
 
         # Create a RefreshToken object that matches what is expected in later code
@@ -139,6 +136,7 @@ class MockOAuthProvider(OAuthAuthorizationServerProvider):
         new_refresh_token = f"refresh_{secrets.token_hex(32)}"
 
         # Store the new tokens
+        assert client.client_id is not None
         self.tokens[new_access_token] = AccessToken(
             token=new_access_token,
             client_id=client.client_id,
@@ -176,17 +174,17 @@ class MockOAuthProvider(OAuthAuthorizationServerProvider):
 
     async def revoke_token(self, token: AccessToken | RefreshToken) -> None:
         match token:
-            case RefreshToken():
+            case RefreshToken():  # pragma: no cover
                 # Remove the refresh token
                 del self.refresh_tokens[token.token]
 
-            case AccessToken():
+            case AccessToken():  # pragma: no branch
                 # Remove the access token
                 del self.tokens[token.token]
 
                 # Also remove any refresh tokens that point to this access token
                 for refresh_token, access_token in list(self.refresh_tokens.items()):
-                    if access_token == token.token:
+                    if access_token == token.token:  # pragma: no branch
                         del self.refresh_tokens[refresh_token]
 
 
@@ -196,7 +194,7 @@ def mock_oauth_provider():
 
 
 @pytest.fixture
-def auth_app(mock_oauth_provider):
+def auth_app(mock_oauth_provider: MockOAuthProvider):
     # Create auth router
     auth_routes = create_auth_routes(
         mock_oauth_provider,
@@ -217,13 +215,15 @@ def auth_app(mock_oauth_provider):
 
 
 @pytest.fixture
-async def test_client(auth_app):
+async def test_client(auth_app: Starlette):
     async with httpx.AsyncClient(transport=httpx.ASGITransport(app=auth_app), base_url="https://mcptest.com") as client:
         yield client
 
 
 @pytest.fixture
-async def registered_client(test_client: httpx.AsyncClient, request):
+async def registered_client(
+    test_client: httpx.AsyncClient, request: pytest.FixtureRequest
+) -> OAuthClientInformationFull:
     """Create and register a test client.
 
     Parameters can be customized via indirect parameterization:
@@ -259,7 +259,12 @@ def pkce_challenge():
 
 
 @pytest.fixture
-async def auth_code(test_client, registered_client, pkce_challenge, request):
+async def auth_code(
+    test_client: httpx.AsyncClient,
+    registered_client: dict[str, Any],
+    pkce_challenge: dict[str, str],
+    request: pytest.FixtureRequest,
+):
     """Get an authorization code.
 
     Parameters can be customized via indirect parameterization:
@@ -278,7 +283,7 @@ async def auth_code(test_client, registered_client, pkce_challenge, request):
     }
 
     # Override with any parameters from the test
-    if hasattr(request, "param") and request.param:
+    if hasattr(request, "param") and request.param:  # pragma: no cover
         auth_params.update(request.param)
 
     response = await test_client.get("/authorize", params=auth_params)
@@ -299,47 +304,12 @@ async def auth_code(test_client, registered_client, pkce_challenge, request):
     }
 
 
-@pytest.fixture
-async def tokens(test_client, registered_client, auth_code, pkce_challenge, request):
-    """Exchange authorization code for tokens.
-
-    Parameters can be customized via indirect parameterization:
-    @pytest.mark.parametrize("tokens",
-                            [{"code_verifier": "wrong_verifier"}],
-                            indirect=True)
-    """
-    # Default token request params
-    token_params = {
-        "grant_type": "authorization_code",
-        "client_id": registered_client["client_id"],
-        "client_secret": registered_client["client_secret"],
-        "code": auth_code["code"],
-        "code_verifier": pkce_challenge["code_verifier"],
-        "redirect_uri": auth_code["redirect_uri"],
-    }
-
-    # Override with any parameters from the test
-    if hasattr(request, "param") and request.param:
-        token_params.update(request.param)
-
-    response = await test_client.post("/token", data=token_params)
-
-    # Don't assert success here since some tests will intentionally cause errors
-    return {
-        "response": response,
-        "params": token_params,
-    }
-
-
 class TestAuthEndpoints:
     @pytest.mark.anyio
     async def test_metadata_endpoint(self, test_client: httpx.AsyncClient):
         """Test the OAuth 2.0 metadata endpoint."""
-        print("Sending request to metadata endpoint")
+
         response = await test_client.get("/.well-known/oauth-authorization-server")
-        print(f"Got response: {response.status_code}")
-        if response.status_code != 200:
-            print(f"Response content: {response.content}")
         assert response.status_code == 200
 
         metadata = response.json()
@@ -373,7 +343,12 @@ class TestAuthEndpoints:
         assert "error_description" in error_response  # Contains validation error messages
 
     @pytest.mark.anyio
-    async def test_token_invalid_auth_code(self, test_client, registered_client, pkce_challenge):
+    async def test_token_invalid_auth_code(
+        self,
+        test_client: httpx.AsyncClient,
+        registered_client: dict[str, Any],
+        pkce_challenge: dict[str, str],
+    ):
         """Test token endpoint error - authorization code does not exist."""
         # Try to use a non-existent authorization code
         response = await test_client.post(
@@ -387,9 +362,7 @@ class TestAuthEndpoints:
                 "redirect_uri": "https://client.example.com/callback",
             },
         )
-        print(f"Status code: {response.status_code}")
-        print(f"Response body: {response.content}")
-        print(f"Response JSON: {response.json()}")
+
         assert response.status_code == 400
         error_response = response.json()
         assert error_response["error"] == "invalid_grant"
@@ -398,11 +371,11 @@ class TestAuthEndpoints:
     @pytest.mark.anyio
     async def test_token_expired_auth_code(
         self,
-        test_client,
-        registered_client,
-        auth_code,
-        pkce_challenge,
-        mock_oauth_provider,
+        test_client: httpx.AsyncClient,
+        registered_client: dict[str, Any],
+        auth_code: dict[str, str],
+        pkce_challenge: dict[str, str],
+        mock_oauth_provider: MockOAuthProvider,
     ):
         """Test token endpoint error - authorization code has expired."""
         # Get the current time for our time mocking
@@ -411,8 +384,8 @@ class TestAuthEndpoints:
         # Find the auth code object
         code_value = auth_code["code"]
         found_code = None
-        for code_obj in mock_oauth_provider.auth_codes.values():
-            if code_obj.code == code_value:
+        for code_obj in mock_oauth_provider.auth_codes.values():  # pragma: no branch
+            if code_obj.code == code_value:  # pragma: no branch
                 found_code = code_obj
                 break
 
@@ -451,7 +424,13 @@ class TestAuthEndpoints:
         ],
         indirect=True,
     )
-    async def test_token_redirect_uri_mismatch(self, test_client, registered_client, auth_code, pkce_challenge):
+    async def test_token_redirect_uri_mismatch(
+        self,
+        test_client: httpx.AsyncClient,
+        registered_client: dict[str, Any],
+        auth_code: dict[str, str],
+        pkce_challenge: dict[str, str],
+    ):
         """Test token endpoint error - redirect URI mismatch."""
         # Try to use the code with a different redirect URI
         response = await test_client.post(
@@ -472,7 +451,9 @@ class TestAuthEndpoints:
         assert "redirect_uri did not match" in error_response["error_description"]
 
     @pytest.mark.anyio
-    async def test_token_code_verifier_mismatch(self, test_client, registered_client, auth_code):
+    async def test_token_code_verifier_mismatch(
+        self, test_client: httpx.AsyncClient, registered_client: dict[str, Any], auth_code: dict[str, str]
+    ):
         """Test token endpoint error - PKCE code verifier mismatch."""
         # Try to use the code with an incorrect code verifier
         response = await test_client.post(
@@ -493,7 +474,7 @@ class TestAuthEndpoints:
         assert "incorrect code_verifier" in error_response["error_description"]
 
     @pytest.mark.anyio
-    async def test_token_invalid_refresh_token(self, test_client, registered_client):
+    async def test_token_invalid_refresh_token(self, test_client: httpx.AsyncClient, registered_client: dict[str, Any]):
         """Test token endpoint error - refresh token does not exist."""
         # Try to use a non-existent refresh token
         response = await test_client.post(
@@ -513,11 +494,10 @@ class TestAuthEndpoints:
     @pytest.mark.anyio
     async def test_token_expired_refresh_token(
         self,
-        test_client,
-        registered_client,
-        auth_code,
-        pkce_challenge,
-        mock_oauth_provider,
+        test_client: httpx.AsyncClient,
+        registered_client: dict[str, Any],
+        auth_code: dict[str, str],
+        pkce_challenge: dict[str, str],
     ):
         """Test token endpoint error - refresh token has expired."""
         # Step 1: First, let's create a token and refresh token at the current time
@@ -560,7 +540,13 @@ class TestAuthEndpoints:
             assert "refresh token has expired" in error_response["error_description"]
 
     @pytest.mark.anyio
-    async def test_token_invalid_scope(self, test_client, registered_client, auth_code, pkce_challenge):
+    async def test_token_invalid_scope(
+        self,
+        test_client: httpx.AsyncClient,
+        registered_client: dict[str, Any],
+        auth_code: dict[str, str],
+        pkce_challenge: dict[str, str],
+    ):
         """Test token endpoint error - invalid scope in refresh token request."""
         # Exchange authorization code for tokens
         token_response = await test_client.post(
@@ -664,8 +650,9 @@ class TestAuthEndpoints:
     @pytest.mark.anyio
     async def test_client_registration_empty_redirect_uris(self, test_client: httpx.AsyncClient):
         """Test client registration with empty redirect_uris array."""
+        redirect_uris: list[str] = []
         client_metadata = {
-            "redirect_uris": [],  # Empty array
+            "redirect_uris": redirect_uris,  # Empty array
             "client_name": "Test Client",
         }
 
@@ -682,12 +669,7 @@ class TestAuthEndpoints:
         )
 
     @pytest.mark.anyio
-    async def test_authorize_form_post(
-        self,
-        test_client: httpx.AsyncClient,
-        mock_oauth_provider: MockOAuthProvider,
-        pkce_challenge,
-    ):
+    async def test_authorize_form_post(self, test_client: httpx.AsyncClient, pkce_challenge: dict[str, str]):
         """Test the authorization endpoint using POST with form-encoded data."""
         # Register a client
         client_metadata = {
@@ -730,7 +712,7 @@ class TestAuthEndpoints:
         self,
         test_client: httpx.AsyncClient,
         mock_oauth_provider: MockOAuthProvider,
-        pkce_challenge,
+        pkce_challenge: dict[str, str],
     ):
         """Test the full authorization flow."""
         # 1. Register a client
@@ -836,7 +818,7 @@ class TestAuthEndpoints:
         assert await mock_oauth_provider.load_access_token(new_token_response["access_token"]) is None
 
     @pytest.mark.anyio
-    async def test_revoke_invalid_token(self, test_client, registered_client):
+    async def test_revoke_invalid_token(self, test_client: httpx.AsyncClient, registered_client: dict[str, Any]):
         """Test revoking an invalid token."""
         response = await test_client.post(
             "/revoke",
@@ -850,7 +832,7 @@ class TestAuthEndpoints:
         assert response.status_code == 200
 
     @pytest.mark.anyio
-    async def test_revoke_with_malformed_token(self, test_client, registered_client):
+    async def test_revoke_with_malformed_token(self, test_client: httpx.AsyncClient, registered_client: dict[str, Any]):
         response = await test_client.post(
             "/revoke",
             data={
@@ -874,10 +856,7 @@ class TestAuthEndpoints:
             "scope": "read write profile admin",  # 'admin' is not in valid_scopes
         }
 
-        response = await test_client.post(
-            "/register",
-            json=client_metadata,
-        )
+        response = await test_client.post("/register", json=client_metadata)
         assert response.status_code == 400
         error_data = response.json()
         assert "error" in error_data
@@ -895,10 +874,7 @@ class TestAuthEndpoints:
             # No scope specified
         }
 
-        response = await test_client.post(
-            "/register",
-            json=client_metadata,
-        )
+        response = await test_client.post("/register", json=client_metadata)
         assert response.status_code == 201
         client_info = response.json()
 
@@ -920,22 +896,92 @@ class TestAuthEndpoints:
             "grant_types": ["authorization_code"],
         }
 
-        response = await test_client.post(
-            "/register",
-            json=client_metadata,
-        )
+        response = await test_client.post("/register", json=client_metadata)
         assert response.status_code == 400
         error_data = response.json()
         assert "error" in error_data
         assert error_data["error"] == "invalid_client_metadata"
         assert error_data["error_description"] == "grant_types must be authorization_code and refresh_token"
 
+    @pytest.mark.anyio
+    async def test_client_registration_with_additional_grant_type(self, test_client: httpx.AsyncClient):
+        client_metadata = {
+            "redirect_uris": ["https://client.example.com/callback"],
+            "client_name": "Test Client",
+            "grant_types": ["authorization_code", "refresh_token", "urn:ietf:params:oauth:grant-type:device_code"],
+        }
+
+        response = await test_client.post("/register", json=client_metadata)
+        assert response.status_code == 201
+        client_info = response.json()
+
+        # Verify client was registered successfully
+        assert "client_id" in client_info
+        assert "client_secret" in client_info
+        assert client_info["client_name"] == "Test Client"
+
+    @pytest.mark.anyio
+    async def test_client_registration_with_additional_response_types(
+        self, test_client: httpx.AsyncClient, mock_oauth_provider: MockOAuthProvider
+    ):
+        """Test that registration accepts additional response_types values alongside 'code'."""
+        client_metadata = {
+            "redirect_uris": ["https://client.example.com/callback"],
+            "client_name": "Test Client",
+            "grant_types": ["authorization_code", "refresh_token"],
+            "response_types": ["code", "none"],  # Keycloak-style response with additional value
+        }
+
+        response = await test_client.post("/register", json=client_metadata)
+        assert response.status_code == 201
+        data = response.json()
+
+        client = await mock_oauth_provider.get_client(data["client_id"])
+        assert client is not None
+        assert "code" in client.response_types
+
+    @pytest.mark.anyio
+    async def test_client_registration_response_types_without_code(self, test_client: httpx.AsyncClient):
+        """Test that registration rejects response_types that don't include 'code'."""
+        client_metadata = {
+            "redirect_uris": ["https://client.example.com/callback"],
+            "client_name": "Test Client",
+            "grant_types": ["authorization_code", "refresh_token"],
+            "response_types": ["token", "none", "nonsense-string"],
+        }
+
+        response = await test_client.post("/register", json=client_metadata)
+        assert response.status_code == 400
+        error_data = response.json()
+        assert "error" in error_data
+        assert error_data["error"] == "invalid_client_metadata"
+        assert "response_types must include 'code'" in error_data["error_description"]
+
+    @pytest.mark.anyio
+    async def test_client_registration_default_response_types(
+        self, test_client: httpx.AsyncClient, mock_oauth_provider: MockOAuthProvider
+    ):
+        """Test that registration uses default response_types of ['code'] when not specified."""
+        client_metadata = {
+            "redirect_uris": ["https://client.example.com/callback"],
+            "client_name": "Test Client",
+            "grant_types": ["authorization_code", "refresh_token"],
+            # response_types not specified, should default to ["code"]
+        }
+
+        response = await test_client.post("/register", json=client_metadata)
+        assert response.status_code == 201
+        data = response.json()
+
+        assert "response_types" in data
+        assert data["response_types"] == ["code"]
+
 
 class TestAuthorizeEndpointErrors:
     """Test error handling in the OAuth authorization endpoint."""
 
     @pytest.mark.anyio
-    async def test_authorize_missing_client_id(self, test_client: httpx.AsyncClient, pkce_challenge):
+    async def test_authorize_missing_client_id(self, test_client: httpx.AsyncClient, pkce_challenge: dict[str, str]):
         """Test authorization endpoint with missing client_id.
 
         According to the OAuth2.0 spec, if client_id is missing, the server should
@@ -959,7 +1005,7 @@ class TestAuthorizeEndpointErrors:
         assert "client_id" in response.text.lower()
 
     @pytest.mark.anyio
-    async def test_authorize_invalid_client_id(self, test_client: httpx.AsyncClient, pkce_challenge):
+    async def test_authorize_invalid_client_id(self, test_client: httpx.AsyncClient, pkce_challenge: dict[str, str]):
         """Test authorization endpoint with invalid client_id.
 
         According to the OAuth2.0 spec, if client_id is invalid, the server should
@@ -984,7 +1030,7 @@ class TestAuthorizeEndpointErrors:
 
     @pytest.mark.anyio
     async def test_authorize_missing_redirect_uri(
-        self, test_client: httpx.AsyncClient, registered_client, pkce_challenge
+        self, test_client: httpx.AsyncClient, registered_client: dict[str, Any], pkce_challenge: dict[str, str]
     ):
         """Test authorization endpoint with missing redirect_uri.
 
@@ -1010,7 +1056,7 @@ class TestAuthorizeEndpointErrors:
 
     @pytest.mark.anyio
     async def test_authorize_invalid_redirect_uri(
-        self, test_client: httpx.AsyncClient, registered_client, pkce_challenge
+        self, test_client: httpx.AsyncClient, registered_client: dict[str, Any], pkce_challenge: dict[str, str]
     ):
         """Test authorization endpoint with invalid redirect_uri.
 
@@ -1050,7 +1096,7 @@ class TestAuthorizeEndpointErrors:
         indirect=True,
     )
     async def test_authorize_missing_redirect_uri_multiple_registered(
-        self, test_client: httpx.AsyncClient, registered_client, pkce_challenge
+        self, test_client: httpx.AsyncClient, registered_client: dict[str, Any], pkce_challenge: dict[str, str]
     ):
         """Test endpoint with missing redirect_uri with multiple registered URIs.
 
@@ -1076,7 +1122,7 @@ class TestAuthorizeEndpointErrors:
 
     @pytest.mark.anyio
     async def test_authorize_unsupported_response_type(
-        self, test_client: httpx.AsyncClient, registered_client, pkce_challenge
+        self, test_client: httpx.AsyncClient, registered_client: dict[str, Any], pkce_challenge: dict[str, str]
     ):
         """Test authorization endpoint with unsupported response_type.
 
@@ -1110,7 +1156,7 @@ class TestAuthorizeEndpointErrors:
 
     @pytest.mark.anyio
     async def test_authorize_missing_response_type(
-        self, test_client: httpx.AsyncClient, registered_client, pkce_challenge
+        self, test_client: httpx.AsyncClient, registered_client: dict[str, Any], pkce_challenge: dict[str, str]
     ):
         """Test authorization endpoint with missing response_type.
 
@@ -1142,7 +1188,9 @@ class TestAuthorizeEndpointErrors:
         assert query_params["state"][0] == "test_state"
 
     @pytest.mark.anyio
-    async def test_authorize_missing_pkce_challenge(self, test_client: httpx.AsyncClient, registered_client):
+    async def test_authorize_missing_pkce_challenge(
+        self, test_client: httpx.AsyncClient, registered_client: dict[str, Any]
+    ):
         """Test authorization endpoint with missing PKCE code_challenge.
 
         Missing PKCE parameters should result in invalid_request error.
@@ -1171,7 +1219,9 @@ class TestAuthorizeEndpointErrors:
         assert query_params["state"][0] == "test_state"
 
     @pytest.mark.anyio
-    async def test_authorize_invalid_scope(self, test_client: httpx.AsyncClient, registered_client, pkce_challenge):
+    async def test_authorize_invalid_scope(
+        self, test_client: httpx.AsyncClient, registered_client: dict[str, Any], pkce_challenge: dict[str, str]
+    ):
         """Test authorization endpoint with invalid scope.
 
         Invalid scope should redirect with invalid_scope error.
