@@ -4,13 +4,8 @@ from typing_extensions import AsyncGenerator
 
 from mcp.client.session import ClientSession
 from mcp.server import Server
-from mcp.shared.memory import (
-    create_connected_server_and_client_session,
-)
-from mcp.types import (
-    EmptyResult,
-    Resource,
-)
+from mcp.shared.memory import create_connected_server_and_client_session
+from mcp.types import EmptyResult, Resource
 
 
 @pytest.fixture
@@ -18,7 +13,7 @@ def mcp_server() -> Server:
     server = Server(name="test_server")
 
     @server.list_resources()
-    async def handle_list_resources():
+    async def handle_list_resources():  # pragma: no cover
         return [
             Resource(
                 uri=AnyUrl("memory://test"),
