@@ -7,7 +7,6 @@ Contains tests for both server and client sides of the StreamableHTTP transport.
 import json
 import multiprocessing
 import socket
-import time
 from collections.abc import Generator
 from typing import Any
 
@@ -40,16 +39,10 @@ from mcp.server.streamable_http_manager import StreamableHTTPSessionManager
 from mcp.server.transport_security import TransportSecuritySettings
 from mcp.shared.context import RequestContext
 from mcp.shared.exceptions import McpError
-from mcp.shared.message import (
-    ClientMessageMetadata,
-)
+from mcp.shared.message import ClientMessageMetadata
 from mcp.shared.session import RequestResponder
-from mcp.types import (
-    InitializeResult,
-    TextContent,
-    TextResourceContents,
-    Tool,
-)
+from mcp.types import InitializeResult, TextContent, TextResourceContents, Tool
+from tests.test_helpers import wait_for_server
 
 # Test constants
 SERVER_NAME = "test_streamable_http_server"
@@ -67,7 +60,7 @@ INIT_REQUEST = {
 
 
 # Helper functions
-def extract_protocol_version_from_sse(response: requests.Response) -> str:
+def extract_protocol_version_from_sse(response: requests.Response) -> str:  # pragma: no cover
     """Extract the negotiated protocol version from an SSE initialization response."""
     assert response.headers.get("Content-Type") == "text/event-stream"
     for line in response.text.splitlines():
@@ -85,14 +78,14 @@ class SimpleEventStore(EventStore):
         self._events: list[tuple[StreamId, EventId, types.JSONRPCMessage]] = []
         self._event_id_counter = 0
 
-    async def store_event(self, stream_id: StreamId, message: types.JSONRPCMessage) -> EventId:
+    async def store_event(self, stream_id: StreamId, message: types.JSONRPCMessage) -> EventId:  # pragma: no cover
         """Store an event and return its ID."""
         self._event_id_counter += 1
         event_id = str(self._event_id_counter)
         self._events.append((stream_id, event_id, message))
         return event_id
 
-    async def replay_events_after(
+    async def replay_events_after(  # pragma: no cover
         self,
         last_event_id: EventId,
         send_callback: EventCallback,
@@ -121,7 +114,7 @@ class SimpleEventStore(EventStore):
 
 
 # Test server implementation that follows MCP protocol
-class ServerTest(Server):
+class ServerTest(Server):  # pragma: no cover
     def __init__(self):
         super().__init__(SERVER_NAME)
         self._lock = None  # Will be initialized in async context
@@ -173,7 +166,7 @@ class ServerTest(Server):
             ]
 
         @self.call_tool()
-        async def handle_call_tool(name: str, args: dict) -> list[TextContent]:
+        async def handle_call_tool(name: str, args: dict[str, Any]) -> list[TextContent]:
             ctx = self.request_context
 
             # When the tool is called, send a notification to test GET stream
@@ -261,7 +254,9 @@ class ServerTest(Server):
             return [TextContent(type="text", text=f"Called {name}")]
 
 
-def create_app(is_json_response_enabled=False, event_store: EventStore | None = None) -> Starlette:
+def create_app(
+    is_json_response_enabled: bool = False, event_store: EventStore | None = None
+) -> Starlette:  # pragma: no cover
     """Create a Starlette application for testing using the session manager.
 
     Args:
@@ -294,7 +289,9 @@ def create_app(is_json_response_enabled=False, event_store: EventStore | None =
     return app
 
 
-def run_server(port: int, is_json_response_enabled=False, event_store: EventStore | None = None) -> None:
+def run_server(
+    port: int, is_json_response_enabled: bool = False, event_store: EventStore | None = None
+) -> None:  # pragma: no cover
     """Run the test server.
 
     Args:
@@ -351,18 +348,7 @@ def basic_server(basic_server_port: int) -> Generator[None, None, None]:
     proc.start()
 
     # Wait for server to be running
-    max_attempts = 20
-    attempt = 0
-    while attempt < max_attempts:
-        try:
-            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
-                s.connect(("127.0.0.1", basic_server_port))
-                break
-        except ConnectionRefusedError:
-            time.sleep(0.1)
-            attempt += 1
-    else:
-        raise RuntimeError(f"Server failed to start after {max_attempts} attempts")
+    wait_for_server(basic_server_port)
 
     yield
 
@@ -398,18 +384,7 @@ def event_server(
     proc.start()
 
     # Wait for server to be running
-    max_attempts = 20
-    attempt = 0
-    while attempt < max_attempts:
-        try:
-            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
-                s.connect(("127.0.0.1", event_server_port))
-                break
-        except ConnectionRefusedError:
-            time.sleep(0.1)
-            attempt += 1
-    else:
-        raise RuntimeError(f"Server failed to start after {max_attempts} attempts")
+    wait_for_server(event_server_port)
 
     yield event_store, f"http://127.0.0.1:{event_server_port}"
 
@@ -429,18 +404,7 @@ def json_response_server(json_server_port: int) -> Generator[None, None, None]:
     proc.start()
 
     # Wait for server to be running
-    max_attempts = 20
-    attempt = 0
-    while attempt < max_attempts:
-        try:
-            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
-                s.connect(("127.0.0.1", json_server_port))
-                break
-        except ConnectionRefusedError:
-            time.sleep(0.1)
-            attempt += 1
-    else:
-        raise RuntimeError(f"Server failed to start after {max_attempts} attempts")
+    wait_for_server(json_server_port)
 
     yield
 
@@ -462,7 +426,7 @@ def json_server_url(json_server_port: int) -> str:
 
 
 # Basic request validation tests
-def test_accept_header_validation(basic_server, basic_server_url):
+def test_accept_header_validation(basic_server: None, basic_server_url: str):
     """Test that Accept header is properly validated."""
     # Test without Accept header
     response = requests.post(
@@ -474,7 +438,7 @@ def test_accept_header_validation(basic_server, basic_server_url):
     assert "Not Acceptable" in response.text
 
 
-def test_content_type_validation(basic_server, basic_server_url):
+def test_content_type_validation(basic_server: None, basic_server_url: str):
     """Test that Content-Type header is properly validated."""
     # Test with incorrect Content-Type
     response = requests.post(
@@ -490,7 +454,7 @@ def test_content_type_validation(basic_server, basic_server_url):
     assert "Invalid Content-Type" in response.text
 
 
-def test_json_validation(basic_server, basic_server_url):
+def test_json_validation(basic_server: None, basic_server_url: str):
     """Test that JSON content is properly validated."""
     # Test with invalid JSON
     response = requests.post(
@@ -505,7 +469,7 @@ def test_json_validation(basic_server, basic_server_url):
     assert "Parse error" in response.text
 
 
-def test_json_parsing(basic_server, basic_server_url):
+def test_json_parsing(basic_server: None, basic_server_url: str):
     """Test that JSON content is properly parse."""
     # Test with valid JSON but invalid JSON-RPC
     response = requests.post(
@@ -520,7 +484,7 @@ def test_json_parsing(basic_server, basic_server_url):
     assert "Validation error" in response.text
 
 
-def test_method_not_allowed(basic_server, basic_server_url):
+def test_method_not_allowed(basic_server: None, basic_server_url: str):
     """Test that unsupported HTTP methods are rejected."""
     # Test with unsupported method (PUT)
     response = requests.put(
@@ -535,7 +499,7 @@ def test_method_not_allowed(basic_server, basic_server_url):
     assert "Method Not Allowed" in response.text
 
 
-def test_session_validation(basic_server, basic_server_url):
+def test_session_validation(basic_server: None, basic_server_url: str):
     """Test session ID validation."""
     # session_id not used directly in this test
 
@@ -610,7 +574,7 @@ def test_streamable_http_transport_init_validation():
         StreamableHTTPServerTransport(mcp_session_id="test\n")
 
 
-def test_session_termination(basic_server, basic_server_url):
+def test_session_termination(basic_server: None, basic_server_url: str):
     """Test session termination via DELETE and subsequent request handling."""
     response = requests.post(
         f"{basic_server_url}/mcp",
@@ -650,7 +614,7 @@ def test_session_termination(basic_server, basic_server_url):
     assert "Session has been terminated" in response.text
 
 
-def test_response(basic_server, basic_server_url):
+def test_response(basic_server: None, basic_server_url: str):
     """Test response handling for a valid request."""
     mcp_url = f"{basic_server_url}/mcp"
     response = requests.post(
@@ -685,7 +649,7 @@ def test_response(basic_server, basic_server_url):
     assert tools_response.headers.get("Content-Type") == "text/event-stream"
 
 
-def test_json_response(json_response_server, json_server_url):
+def test_json_response(json_response_server: None, json_server_url: str):
     """Test response handling when is_json_response_enabled is True."""
     mcp_url = f"{json_server_url}/mcp"
     response = requests.post(
@@ -700,7 +664,52 @@ def test_json_response(json_response_server, json_server_url):
     assert response.headers.get("Content-Type") == "application/json"
 
 
-def test_get_sse_stream(basic_server, basic_server_url):
+def test_json_response_accept_json_only(json_response_server: None, json_server_url: str):
+    """Test that json_response servers only require application/json in Accept header."""
+    mcp_url = f"{json_server_url}/mcp"
+    response = requests.post(
+        mcp_url,
+        headers={
+            "Accept": "application/json",
+            "Content-Type": "application/json",
+        },
+        json=INIT_REQUEST,
+    )
+    assert response.status_code == 200
+    assert response.headers.get("Content-Type") == "application/json"
+
+
+def test_json_response_missing_accept_header(json_response_server: None, json_server_url: str):
+    """Test that json_response servers reject requests without Accept header."""
+    mcp_url = f"{json_server_url}/mcp"
+    response = requests.post(
+        mcp_url,
+        headers={
+            "Content-Type": "application/json",
+        },
+        json=INIT_REQUEST,
+    )
+    assert response.status_code == 406
+    assert "Not Acceptable" in response.text
+
+
+def test_json_response_incorrect_accept_header(json_response_server: None, json_server_url: str):
+    """Test that json_response servers reject requests with incorrect Accept header."""
+    mcp_url = f"{json_server_url}/mcp"
+    # Test with only text/event-stream (wrong for JSON server)
+    response = requests.post(
+        mcp_url,
+        headers={
+            "Accept": "text/event-stream",
+            "Content-Type": "application/json",
+        },
+        json=INIT_REQUEST,
+    )
+    assert response.status_code == 406
+    assert "Not Acceptable" in response.text
+
+
+def test_get_sse_stream(basic_server: None, basic_server_url: str):
     """Test establishing an SSE stream via GET request."""
     # First, we need to initialize a session
     mcp_url = f"{basic_server_url}/mcp"
@@ -721,8 +730,8 @@ def test_get_sse_stream(basic_server, basic_server_url):
     # Extract negotiated protocol version from SSE response
     init_data = None
     assert init_response.headers.get("Content-Type") == "text/event-stream"
-    for line in init_response.text.splitlines():
-        if line.startswith("data: "):
+    for line in init_response.text.splitlines():  # pragma: no branch
+        if line.startswith("data: "):  # pragma: no cover
             init_data = json.loads(line[6:])
             break
     assert init_data is not None
@@ -760,7 +769,7 @@ def test_get_sse_stream(basic_server, basic_server_url):
     assert second_get.status_code == 409
 
 
-def test_get_validation(basic_server, basic_server_url):
+def test_get_validation(basic_server: None, basic_server_url: str):
     """Test validation for GET requests."""
     # First, we need to initialize a session
     mcp_url = f"{basic_server_url}/mcp"
@@ -781,8 +790,8 @@ def test_get_validation(basic_server, basic_server_url):
     # Extract negotiated protocol version from SSE response
     init_data = None
     assert init_response.headers.get("Content-Type") == "text/event-stream"
-    for line in init_response.text.splitlines():
-        if line.startswith("data: "):
+    for line in init_response.text.splitlines():  # pragma: no branch
+        if line.startswith("data: "):  # pragma: no cover
             init_data = json.loads(line[6:])
             break
     assert init_data is not None
@@ -815,14 +824,14 @@ def test_get_validation(basic_server, basic_server_url):
 
 # Client-specific fixtures
 @pytest.fixture
-async def http_client(basic_server, basic_server_url):
+async def http_client(basic_server: None, basic_server_url: str):  # pragma: no cover
     """Create test client matching the SSE test pattern."""
     async with httpx.AsyncClient(base_url=basic_server_url) as client:
         yield client
 
 
 @pytest.fixture
-async def initialized_client_session(basic_server, basic_server_url):
+async def initialized_client_session(basic_server: None, basic_server_url: str):
     """Create initialized StreamableHTTP client session."""
     async with streamablehttp_client(f"{basic_server_url}/mcp") as (
         read_stream,
@@ -838,7 +847,7 @@ async def initialized_client_session(basic_server, basic_server_url):
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_basic_connection(basic_server, basic_server_url):
+async def test_streamablehttp_client_basic_connection(basic_server: None, basic_server_url: str):
     """Test basic client connection with initialization."""
     async with streamablehttp_client(f"{basic_server_url}/mcp") as (
         read_stream,
@@ -856,16 +865,17 @@ async def test_streamablehttp_client_basic_connection(basic_server, basic_server
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_resource_read(initialized_client_session):
+async def test_streamablehttp_client_resource_read(initialized_client_session: ClientSession):
     """Test client resource read functionality."""
     response = await initialized_client_session.read_resource(uri=AnyUrl("foobar://test-resource"))
     assert len(response.contents) == 1
     assert response.contents[0].uri == AnyUrl("foobar://test-resource")
+    assert isinstance(response.contents[0], TextResourceContents)
     assert response.contents[0].text == "Read test-resource"
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_tool_invocation(initialized_client_session):
+async def test_streamablehttp_client_tool_invocation(initialized_client_session: ClientSession):
     """Test client tool invocation."""
     # First list tools
     tools = await initialized_client_session.list_tools()
@@ -880,7 +890,7 @@ async def test_streamablehttp_client_tool_invocation(initialized_client_session)
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_error_handling(initialized_client_session):
+async def test_streamablehttp_client_error_handling(initialized_client_session: ClientSession):
     """Test error handling in client."""
     with pytest.raises(McpError) as exc_info:
         await initialized_client_session.read_resource(uri=AnyUrl("unknown://test-error"))
@@ -889,7 +899,7 @@ async def test_streamablehttp_client_error_handling(initialized_client_session):
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_session_persistence(basic_server, basic_server_url):
+async def test_streamablehttp_client_session_persistence(basic_server: None, basic_server_url: str):
     """Test that session ID persists across requests."""
     async with streamablehttp_client(f"{basic_server_url}/mcp") as (
         read_stream,
@@ -917,7 +927,7 @@ async def test_streamablehttp_client_session_persistence(basic_server, basic_ser
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_json_response(json_response_server, json_server_url):
+async def test_streamablehttp_client_json_response(json_response_server: None, json_server_url: str):
     """Test client with JSON response mode."""
     async with streamablehttp_client(f"{json_server_url}/mcp") as (
         read_stream,
@@ -945,18 +955,18 @@ async def test_streamablehttp_client_json_response(json_response_server, json_se
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_get_stream(basic_server, basic_server_url):
+async def test_streamablehttp_client_get_stream(basic_server: None, basic_server_url: str):
     """Test GET stream functionality for server-initiated messages."""
     import mcp.types as types
     from mcp.shared.session import RequestResponder
 
-    notifications_received = []
+    notifications_received: list[types.ServerNotification] = []
 
     # Define message handler to capture notifications
-    async def message_handler(
+    async def message_handler(  # pragma: no branch
         message: RequestResponder[types.ServerRequest, types.ClientResult] | types.ServerNotification | Exception,
     ) -> None:
-        if isinstance(message, types.ServerNotification):
+        if isinstance(message, types.ServerNotification):  # pragma: no branch
             notifications_received.append(message)
 
     async with streamablehttp_client(f"{basic_server_url}/mcp") as (
@@ -978,7 +988,7 @@ async def test_streamablehttp_client_get_stream(basic_server, basic_server_url):
             # Verify the notification is a ResourceUpdatedNotification
             resource_update_found = False
             for notif in notifications_received:
-                if isinstance(notif.root, types.ResourceUpdatedNotification):
+                if isinstance(notif.root, types.ResourceUpdatedNotification):  # pragma: no branch
                     assert str(notif.root.params.uri) == "http://test_resource/"
                     resource_update_found = True
 
@@ -986,7 +996,7 @@ async def test_streamablehttp_client_get_stream(basic_server, basic_server_url):
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_session_termination(basic_server, basic_server_url):
+async def test_streamablehttp_client_session_termination(basic_server: None, basic_server_url: str):
     """Test client session termination functionality."""
 
     captured_session_id = None
@@ -1008,8 +1018,8 @@ async def test_streamablehttp_client_session_termination(basic_server, basic_ser
             tools = await session.list_tools()
             assert len(tools.tools) == 6
 
-    headers = {}
-    if captured_session_id:
+    headers: dict[str, str] = {}  # pragma: no cover
+    if captured_session_id:  # pragma: no cover
         headers[MCP_SESSION_ID_HEADER] = captured_session_id
 
     async with streamablehttp_client(f"{basic_server_url}/mcp", headers=headers) as (
@@ -1019,7 +1029,7 @@ async def test_streamablehttp_client_session_termination(basic_server, basic_ser
     ):
         async with ClientSession(read_stream, write_stream) as session:
             # Attempt to make a request after termination
-            with pytest.raises(
+            with pytest.raises(  # pragma: no branch
                 McpError,
                 match="Session terminated",
             ):
@@ -1027,7 +1037,9 @@ async def test_streamablehttp_client_session_termination(basic_server, basic_ser
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_session_termination_204(basic_server, basic_server_url, monkeypatch):
+async def test_streamablehttp_client_session_termination_204(
+    basic_server: None, basic_server_url: str, monkeypatch: pytest.MonkeyPatch
+):
     """Test client session termination functionality with a 204 response.
 
     This test patches the httpx client to return a 204 response for DELETEs.
@@ -1037,7 +1049,7 @@ async def test_streamablehttp_client_session_termination_204(basic_server, basic
     original_delete = httpx.AsyncClient.delete
 
     # Mock the client's delete method to return a 204
-    async def mock_delete(self, *args, **kwargs):
+    async def mock_delete(self: httpx.AsyncClient, *args: Any, **kwargs: Any) -> httpx.Response:
         # Call the original method to get the real response
         response = await original_delete(self, *args, **kwargs)
 
@@ -1072,8 +1084,8 @@ async def test_streamablehttp_client_session_termination_204(basic_server, basic
             tools = await session.list_tools()
             assert len(tools.tools) == 6
 
-    headers = {}
-    if captured_session_id:
+    headers: dict[str, str] = {}  # pragma: no cover
+    if captured_session_id:  # pragma: no cover
         headers[MCP_SESSION_ID_HEADER] = captured_session_id
 
     async with streamablehttp_client(f"{basic_server_url}/mcp", headers=headers) as (
@@ -1083,7 +1095,7 @@ async def test_streamablehttp_client_session_termination_204(basic_server, basic
     ):
         async with ClientSession(read_stream, write_stream) as session:
             # Attempt to make a request after termination
-            with pytest.raises(
+            with pytest.raises(  # pragma: no branch
                 McpError,
                 match="Session terminated",
             ):
@@ -1091,24 +1103,24 @@ async def test_streamablehttp_client_session_termination_204(basic_server, basic
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_client_resumption(event_server):
+async def test_streamablehttp_client_resumption(event_server: tuple[SimpleEventStore, str]):
     """Test client session resumption using sync primitives for reliable coordination."""
     _, server_url = event_server
 
     # Variables to track the state
     captured_session_id = None
     captured_resumption_token = None
-    captured_notifications = []
+    captured_notifications: list[types.ServerNotification] = []
     captured_protocol_version = None
     first_notification_received = False
 
-    async def message_handler(
+    async def message_handler(  # pragma: no branch
         message: RequestResponder[types.ServerRequest, types.ClientResult] | types.ServerNotification | Exception,
     ) -> None:
-        if isinstance(message, types.ServerNotification):
+        if isinstance(message, types.ServerNotification):  # pragma: no branch
             captured_notifications.append(message)
             # Look for our first notification
-            if isinstance(message.root, types.LoggingMessageNotification):
+            if isinstance(message.root, types.LoggingMessageNotification):  # pragma: no branch
                 if message.root.params.data == "First notification before lock":
                     nonlocal first_notification_received
                     first_notification_received = True
@@ -1142,7 +1154,6 @@ async def test_streamablehttp_client_resumption(event_server):
                     await session.send_request(
                         types.ClientRequest(
                             types.CallToolRequest(
-                                method="tools/call",
                                 params=types.CallToolRequestParams(
                                     name="wait_for_lock_with_notification", arguments={}
                                 ),
@@ -1162,18 +1173,18 @@ async def test_streamablehttp_client_resumption(event_server):
                 tg.cancel_scope.cancel()
 
     # Verify we received exactly one notification
-    assert len(captured_notifications) == 1
-    assert isinstance(captured_notifications[0].root, types.LoggingMessageNotification)
-    assert captured_notifications[0].root.params.data == "First notification before lock"
+    assert len(captured_notifications) == 1  # pragma: no cover
+    assert isinstance(captured_notifications[0].root, types.LoggingMessageNotification)  # pragma: no cover
+    assert captured_notifications[0].root.params.data == "First notification before lock"  # pragma: no cover
 
     # Clear notifications for the second phase
-    captured_notifications = []
+    captured_notifications = []  # pragma: no cover
 
     # Now resume the session with the same mcp-session-id and protocol version
-    headers = {}
-    if captured_session_id:
+    headers: dict[str, Any] = {}  # pragma: no cover
+    if captured_session_id:  # pragma: no cover
         headers[MCP_SESSION_ID_HEADER] = captured_session_id
-    if captured_protocol_version:
+    if captured_protocol_version:  # pragma: no cover
         headers[MCP_PROTOCOL_VERSION_HEADER] = captured_protocol_version
     async with streamablehttp_client(f"{server_url}/mcp", headers=headers) as (
         read_stream,
@@ -1184,7 +1195,6 @@ async def test_streamablehttp_client_resumption(event_server):
             result = await session.send_request(
                 types.ClientRequest(
                     types.CallToolRequest(
-                        method="tools/call",
                         params=types.CallToolRequestParams(name="release_lock", arguments={}),
                     )
                 ),
@@ -1197,7 +1207,6 @@ async def test_streamablehttp_client_resumption(event_server):
             result = await session.send_request(
                 types.ClientRequest(
                     types.CallToolRequest(
-                        method="tools/call",
                         params=types.CallToolRequestParams(name="wait_for_lock_with_notification", arguments={}),
                     )
                 ),
@@ -1211,11 +1220,12 @@ async def test_streamablehttp_client_resumption(event_server):
             # We should have received the remaining notifications
             assert len(captured_notifications) == 1
 
+            assert isinstance(captured_notifications[0].root, types.LoggingMessageNotification)
             assert captured_notifications[0].root.params.data == "Second notification after lock"
 
 
 @pytest.mark.anyio
-async def test_streamablehttp_server_sampling(basic_server, basic_server_url):
+async def test_streamablehttp_server_sampling(basic_server: None, basic_server_url: str):
     """Test server-initiated sampling request through streamable HTTP transport."""
     # Variable to track if sampling callback was invoked
     sampling_callback_invoked = False
@@ -1272,7 +1282,7 @@ async def test_streamablehttp_server_sampling(basic_server, basic_server_url):
 
 
 # Context-aware server implementation for testing request context propagation
-class ContextAwareServerTest(Server):
+class ContextAwareServerTest(Server):  # pragma: no cover
     def __init__(self):
         super().__init__("ContextAwareServer")
 
@@ -1298,7 +1308,7 @@ class ContextAwareServerTest(Server):
             ]
 
         @self.call_tool()
-        async def handle_call_tool(name: str, args: dict) -> list[TextContent]:
+        async def handle_call_tool(name: str, args: dict[str, Any]) -> list[TextContent]:
             ctx = self.request_context
 
             if name == "echo_headers":
@@ -1306,16 +1316,11 @@ class ContextAwareServerTest(Server):
                 headers_info = {}
                 if ctx.request and isinstance(ctx.request, Request):
                     headers_info = dict(ctx.request.headers)
-                return [
-                    TextContent(
-                        type="text",
-                        text=json.dumps(headers_info),
-                    )
-                ]
+                return [TextContent(type="text", text=json.dumps(headers_info))]
 
             elif name == "echo_context":
                 # Return full context information
-                context_data = {
+                context_data: dict[str, Any] = {
                     "request_id": args.get("request_id"),
                     "headers": {},
                     "method": None,
@@ -1337,7 +1342,7 @@ class ContextAwareServerTest(Server):
 
 
 # Server runner for context-aware testing
-def run_context_aware_server(port: int):
+def run_context_aware_server(port: int):  # pragma: no cover
     """Run the context-aware test server."""
     server = ContextAwareServerTest()
 
@@ -1373,24 +1378,13 @@ def context_aware_server(basic_server_port: int) -> Generator[None, None, None]:
     proc.start()
 
     # Wait for server to be running
-    max_attempts = 20
-    attempt = 0
-    while attempt < max_attempts:
-        try:
-            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
-                s.connect(("127.0.0.1", basic_server_port))
-                break
-        except ConnectionRefusedError:
-            time.sleep(0.1)
-            attempt += 1
-    else:
-        raise RuntimeError(f"Context-aware server failed to start after {max_attempts} attempts")
+    wait_for_server(basic_server_port)
 
     yield
 
     proc.kill()
     proc.join(timeout=2)
-    if proc.is_alive():
+    if proc.is_alive():  # pragma: no cover
         print("Context-aware server process failed to terminate")
 
 
@@ -1430,7 +1424,7 @@ async def test_streamablehttp_request_context_propagation(context_aware_server:
 @pytest.mark.anyio
 async def test_streamablehttp_request_context_isolation(context_aware_server: None, basic_server_url: str) -> None:
     """Test that request contexts are isolated between StreamableHTTP clients."""
-    contexts = []
+    contexts: list[dict[str, Any]] = []
 
     # Create multiple clients with different headers
     for i in range(3):
@@ -1453,8 +1447,8 @@ async def test_streamablehttp_request_context_isolation(context_aware_server: No
                 contexts.append(context_data)
 
     # Verify each request had its own context
-    assert len(contexts) == 3
-    for i, ctx in enumerate(contexts):
+    assert len(contexts) == 3  # pragma: no cover
+    for i, ctx in enumerate(contexts):  # pragma: no cover
         assert ctx["request_id"] == f"request-{i}"
         assert ctx["headers"].get("x-request-id") == f"request-{i}"
         assert ctx["headers"].get("x-custom-value") == f"value-{i}"
@@ -1462,7 +1456,7 @@ async def test_streamablehttp_request_context_isolation(context_aware_server: No
 
 
 @pytest.mark.anyio
-async def test_client_includes_protocol_version_header_after_init(context_aware_server, basic_server_url):
+async def test_client_includes_protocol_version_header_after_init(context_aware_server: None, basic_server_url: str):
     """Test that client includes mcp-protocol-version header after initialization."""
     async with streamablehttp_client(f"{basic_server_url}/mcp") as (
         read_stream,
@@ -1486,7 +1480,7 @@ async def test_client_includes_protocol_version_header_after_init(context_aware_
             assert headers_data[MCP_PROTOCOL_VERSION_HEADER] == negotiated_version
 
 
-def test_server_validates_protocol_version_header(basic_server, basic_server_url):
+def test_server_validates_protocol_version_header(basic_server: None, basic_server_url: str):
     """Test that server returns 400 Bad Request version if header unsupported or invalid."""
     # First initialize a session to get a valid session ID
     init_response = requests.post(
@@ -1544,7 +1538,7 @@ def test_server_validates_protocol_version_header(basic_server, basic_server_url
     assert response.status_code == 200
 
 
-def test_server_backwards_compatibility_no_protocol_version(basic_server, basic_server_url):
+def test_server_backwards_compatibility_no_protocol_version(basic_server: None, basic_server_url: str):
     """Test server accepts requests without protocol version header."""
     # First initialize a session to get a valid session ID
     init_response = requests.post(
@@ -1574,7 +1568,7 @@ def test_server_backwards_compatibility_no_protocol_version(basic_server, basic_
 
 
 @pytest.mark.anyio
-async def test_client_crash_handled(basic_server, basic_server_url):
+async def test_client_crash_handled(basic_server: None, basic_server_url: str):
     """Test that cases where the client crashes are handled gracefully."""
 
     # Simulate bad client that crashes after init
